package com.ynet.life.search.core;

import com.ynet.life.base.Trace;
import com.ynet.life.exception.TranFailException;
import java.io.IOException;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Filter;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.Sort;
import org.apache.lucene.search.TopDocs;
import org.wltea.analyzer.lucene.IKQueryParser;

/* loaded from: com_ynet_life_search_core.jar:com/ynet/life/search/core/IndexSearchImpl.class */
public class IndexSearchImpl implements IndexSearch {
    private final Searcher indexSearcher;
    private final Analyzer analyzer;
    private Filter filter;
    private Sort sorter;

    public IndexSearchImpl(IndexSource indexSource) throws TranFailException {
        this(indexSource, null, null);
    }

    public IndexSearchImpl(IndexSource indexSource, Filter filter, Sort sorter) throws TranFailException {
        this.analyzer = indexSource.getAnalyzer();
        this.indexSearcher = indexSource.createIndexSearcher();
        this.filter = filter;
        this.sorter = sorter;
    }

    @Override // com.ynet.life.search.core.IndexSearch
    public SearchResult search(String queryString, int howMany) throws TranFailException {
        QueryStringParser queryStringParser = new QueryStringParser();
        queryStringParser.parse(queryString);
        String[] fields = queryStringParser.getFields();
        String[] queries = queryStringParser.getQueries();
        BooleanClause.Occur[] occurs = queryStringParser.getOccurs();
        BooleanQuery boolQuery = new BooleanQuery();
        for (int i = 0; i < fields.length; i++) {
            try {
                Query query = null;
                if (queries[i].indexOf(",") != -1) {
                    String[] queryArray = queries[i].split(",");
                    String[] fieldArray = new String[queryArray.length];
                    BooleanClause.Occur[] flagArray = new BooleanClause.Occur[queryArray.length];
                    String queryStr = "";
                    int j = 0;
                    while (j < queryArray.length) {
                        queryArray[j] = String.valueOf(queryArray[j]) + "*";
                        fieldArray[j] = fields[i];
                        flagArray[j] = BooleanClause.Occur.SHOULD;
                        queryStr = j == 0 ? String.valueOf(queryStr) + queryArray[j] : String.valueOf(queryStr) + " " + fieldArray[j] + ":" + queryArray[j];
                        j++;
                    }
                    Query queryParser = new QueryParser(fieldArray[0], new StandardAnalyzer());
                    try {
                        queryParser = queryParser.parse(queryStr);
                        query = queryParser;
                    } catch (ParseException e) {
                        queryParser.printStackTrace();
                    }
                } else {
                    query = IKQueryParser.parse(fields[i], queries[i]);
                }
                boolQuery.add(query, occurs[i]);
            } catch (IOException ioe) {
                Trace.logError("LifeSearch", String.format("查询分析失败: '%s'", ioe.getMessage()), ioe);
                throw new TranFailException(ioe);
            }
        }
        return doSearch(boolQuery, howMany);
    }

    private SearchResult doSearch(Query query, int howMany) throws TranFailException {
        TopDocs topDocs;
        try {
            if (this.filter == null) {
                topDocs = this.indexSearcher.search(query, howMany);
            } else if (this.sorter == null) {
                topDocs = this.indexSearcher.search(query, this.filter, howMany);
            } else {
                topDocs = this.indexSearcher.search(query, this.filter, howMany, this.sorter);
            }
            ScoreDoc[] scores = topDocs.scoreDocs;
            SearchResult searchResult = new SearchResult();
            searchResult.length = scores.length;
            searchResult.totalHits = topDocs.totalHits;
            for (ScoreDoc scoreDoc : scores) {
                int docId = scoreDoc.doc;
                try {
                    searchResult.add(this.indexSearcher.doc(docId));
                } catch (IOException ioe) {
                    Trace.logError("LifeSearch", String.format("搜索时访问索引失败: '%s'", ioe.getMessage()), ioe);
                    throw new TranFailException(ioe);
                } catch (CorruptIndexException cie) {
                    Trace.logError("LifeSearch", String.format("索引已损坏: '%s'", cie.getMessage()), cie);
                    throw new TranFailException(cie);
                }
            }
            return searchResult;
        } catch (IOException ioe2) {
            Trace.logError("LifeSearch", String.format("搜索时访问索引失败: '%s'", ioe2.getMessage()), ioe2);
            throw new TranFailException(ioe2);
        }
    }

    @Override // com.ynet.life.search.core.IndexSearch
    public void close() throws TranFailException {
        try {
            this.indexSearcher.close();
        } catch (IOException ioe) {
            Trace.logError("LifeSearch", String.format("关闭IndexSearcher失败: '%s'", ioe.getMessage()), ioe);
            throw new TranFailException(ioe);
        }
    }
}
